@@ -39,6 +39,11 @@ TEST_PROJ_MAX = 1
 TEST_PROJ_INVESTIGATOR = "Smith"
 TEST_PROJ_NUM = 1
 
+SPECIAL_CHARACTERS = '`~!@#$%^&*()_+-={}|[]\\;:"\',./<>?'
+INVALID_IDENTIFIER, INVALID_IDENTIFIER_FIXED = '9 %$ ', '_9____'
+RESERVED_IDENTIFIER = 'column_definitions'
+CONFLICTING_IDENTIFIER, CONFLICTING_IDENTIFIER_FIXED = RESERVED_IDENTIFIER + '1', RESERVED_IDENTIFIER + '2'
+
 hostname = os.getenv("DERIVA_PY_TEST_HOSTNAME")
 logger = logging.getLogger(__name__)
 if os.getenv("DERIVA_PY_TEST_VERBOSE"):
@@ -63,7 +68,10 @@ def define_test_schema(catalog):
         column_defs=[
             _em.Column.define(cname, ctype) for (cname, ctype) in [
                 ('Investigator', _em.builtin_types.text),
-                ('Num', _em.builtin_types.int4)
+                ('Num', _em.builtin_types.int4),
+                (INVALID_IDENTIFIER, _em.builtin_types.int4),
+                (RESERVED_IDENTIFIER, _em.builtin_types.text),
+                (RESERVED_IDENTIFIER + '1', _em.builtin_types.text)
             ]
         ],
         key_defs=[
@@ -81,7 +89,7 @@ def define_test_schema(catalog):
                 ('Amount', _em.builtin_types.int4),
                 ('Time', _em.builtin_types.timestamptz),
                 ('Type', _em.builtin_types.text),
-                ('Project_Investigator', _em.builtin_types.text),
+                ('Project Investigator', _em.builtin_types.text),
                 ('Project_Num', _em.builtin_types.int4),
                 ('Empty', _em.builtin_types.int4)
             ]
@@ -91,7 +99,7 @@ def define_test_schema(catalog):
         ],
         fkey_defs=[
             _em.ForeignKey.define(['Type'], 'Vocab', 'Experiment_Type', ['ID']),
-            _em.ForeignKey.define(['Project_Investigator', 'Project_Num'], 'ISA', 'Project', ['Investigator', 'Num'])
+            _em.ForeignKey.define(['Project Investigator', 'Project_Num'], 'ISA', 'Project', ['Investigator', 'Num'])
         ]
     )
     isa.create_table(table_def)
@@ -114,7 +122,7 @@ def _generate_experiment_entities(types, count):
             "Amount": i,
             "Time": "2018-01-{}T01:00:00.0".format(1 + (i % 31)),
             "Type": types[i % TEST_EXPTYPE_MAX]['ID'],
-            "Project_Investigator": TEST_PROJ_INVESTIGATOR,
+            "Project Investigator": TEST_PROJ_INVESTIGATOR,
             "Project_Num": TEST_PROJ_NUM,
             "Empty": None
         }
@@ -184,16 +192,15 @@ class DatapathTests (unittest.TestCase):
         self.assertIn('schemas', dir(self.paths))
 
     def test_schema_dir_base(self):
-        self.assertLess({'name', 'tables', 'describe'}, set(dir(self.paths.schemas['ISA'])))
+        self.assertLess({'_name', 'tables', 'describe'}, set(dir(self.paths.schemas['ISA'])))
 
     def test_datapath_dir_base(self):
         self.assertLess({'aggregates', 'groupby', 'attributes', 'context', 'delete', 'entities', 'filter',
                          'link', 'table_instances', 'uri'}, set(dir(self.paths.schemas['ISA'].tables['Experiment'].path)))
 
     def test_table_dir_base(self):
-        self.assertLess({'aggregates', 'alias', 'groupby', 'attributes', 'catalog', 'describe', 'entities',
-                         'filter', 'fqname', 'fromname', 'insert', 'link', 'name', 'path', 'sname',
-                         'uname', 'update', 'uri'}, set(dir(self.paths.schemas['ISA'].tables['Experiment'])))
+        self.assertLess({'aggregates', 'alias', 'groupby', 'attributes', 'describe', 'entities', 'filter', 'insert',
+                         'link', 'path', 'update', 'uri'}, set(dir(self.paths.schemas['ISA'].tables['Experiment'])))
 
     def test_catalog_dir_with_schemas(self):
         self.assertLess({'ISA', 'Vocab'}, set(dir(self.paths)))
@@ -207,14 +214,26 @@ class DatapathTests (unittest.TestCase):
     def test_dir_path(self):
         self.assertIn('Experiment', dir(self.paths.ISA.Experiment.path))
 
+    def test_dir_invalid_identifier(self):
+        self.assertIn(INVALID_IDENTIFIER_FIXED, dir(self.project))
+        self.assertIsNotNone(getattr(self.project, INVALID_IDENTIFIER_FIXED))
+
+    def test_dir_conflicting_identifier(self):
+        self.assertIn(CONFLICTING_IDENTIFIER_FIXED, dir(self.project))
+        self.assertIsNotNone(getattr(self.project, CONFLICTING_IDENTIFIER))
+        self.assertIsNotNone(getattr(self.project, CONFLICTING_IDENTIFIER_FIXED))
+
     def test_describe_schema(self):
-        self.assertTrue(self.paths.schemas['ISA'].describe())
+        with self.assertWarns(DeprecationWarning):
+            self.paths.schemas['ISA'].describe()
 
     def test_describe_table(self):
-        self.assertTrue(self.paths.schemas['ISA'].tables['Experiment'].describe())
+        with self.assertWarns(DeprecationWarning):
+            self.paths.schemas['ISA'].tables['Experiment'].describe()
 
     def test_describe_column(self):
-        self.assertTrue(self.paths.schemas['ISA'].tables['Experiment'].column_definitions['Name'].describe())
+        with self.assertWarns(DeprecationWarning):
+            self.paths.schemas['ISA'].tables['Experiment'].column_definitions['Name'].describe()
 
     def test_unfiltered_fetch(self):
         results = self.experiment.entities()
@@ -280,9 +299,9 @@ class DatapathTests (unittest.TestCase):
         self.assertIn('Amount', result)
 
     def test_attribute_err_table_attr(self):
-        table_attr = ['name', 'sname', 'catalog']
+        table_attr = ['_name', '_schema']
         for attr in table_attr:
-            with self.assertRaises(ValueError):
+            with self.assertRaises(TypeError):
                 self.experiment.attributes(getattr(self.experiment, attr))
 
     def test_update_err_no_targets(self):
@@ -291,11 +310,11 @@ class DatapathTests (unittest.TestCase):
             self.experiment.update(entities)
 
     def test_aggregate_w_invalid_attributes(self):
-        with self.assertRaises(ValueError):
+        with self.assertRaises(TypeError):
             self.experiment.aggregates(Min(self.experiment.column_definitions['Amount']))
 
     def test_aggregate_w_invalid_renames(self):
-        with self.assertRaises(ValueError):
+        with self.assertRaises(TypeError):
             self.experiment.aggregates(
                 self.experiment.column_definitions['Name'],
                 Min(self.experiment.column_definitions['Amount'])
@@ -394,7 +413,7 @@ class DatapathTests (unittest.TestCase):
 
                 result = results[0]
                 self.assertEqual(len(results), TEST_EXPTYPE_MAX)
-                self.assertTrue(all(key.name in result for key in group_key))
+                self.assertTrue(all(key._name in result for key in group_key))
                 self.assertIn(name, result)
                 self.assertEqual(result[name], value)
 
@@ -433,7 +452,7 @@ class DatapathTests (unittest.TestCase):
                 results = self.experiment.groupby(*group_key).attributes(
                     Fn(self.experiment.column_definitions['Amount']).alias(name)).fetch()
 
-                self.assertTrue(all(key.name in results[0] for key in group_key))
+                self.assertTrue(all(key._name in results[0] for key in group_key))
                 self.assertIn(name, results[0])
                 for result in results:
                     bin = result[bin_name]
@@ -465,7 +484,7 @@ class DatapathTests (unittest.TestCase):
                 results = self.experiment.groupby(bin).attributes(
                     Fn(self.experiment.column_definitions['Amount']).alias(name)).sort(sort_key).fetch()
 
-                self.assertIn(bin.name, results[0])
+                self.assertIn(bin._name, results[0])
                 self.assertIn(name, results[0])
                 self.assertTrue(compfn(name, results[0], results[1]))
 
@@ -566,13 +585,13 @@ class DatapathTests (unittest.TestCase):
 
     def test_attribute_rename_special_chars(self):
         # first test with only the `:` character present which would trigger a lexical error from ermrest
-        special_character_out_alias = self.experiment.name + ':' + self.experiment.column_definitions['Name'].name
+        special_character_out_alias = self.experiment._name + ':' + self.experiment.column_definitions['Name']._name
         results = self.experiment.attributes(self.experiment.column_definitions['Name'].alias(special_character_out_alias))
         result = results.fetch(limit=1)[0]
         self.assertIn(special_character_out_alias, result)
 
         # second test with url unsafe characters present which would trigger a bad request from the web server
-        special_character_out_alias = '`~!@#$%^&*()_+-={}|[]\\;:"\',./<>?'
+        special_character_out_alias = SPECIAL_CHARACTERS
         results = self.experiment.attributes(self.experiment.column_definitions['Name'].alias(special_character_out_alias))
         result = results.fetch(limit=1)[0]
         self.assertIn(special_character_out_alias, result)
@@ -613,13 +632,13 @@ class DatapathTests (unittest.TestCase):
         self.assertEqual(len(results), 0)
 
     def test_insert_entities_not_iterable(self):
-        with self.assertRaises(ValueError):
+        with self.assertRaises(TypeError):
             self.experiment_type.insert(1)
 
     def test_insert_entities0_not_dict(self):
-        with self.assertRaises(ValueError):
+        with self.assertRaises(TypeError):
             self.experiment_type.insert([1])
-        with self.assertRaises(ValueError):
+        with self.assertRaises(TypeError):
             self.experiment_type.insert('this is not a dict')
 
     def test_insert(self):
@@ -644,13 +663,13 @@ class DatapathTests (unittest.TestCase):
         self.assertEqual(len(results), 0)
 
     def test_update_entities_not_iterable(self):
-        with self.assertRaises(ValueError):
+        with self.assertRaises(TypeError):
             self.experiment_type.update(1)
 
     def test_update_entities0_not_dict(self):
-        with self.assertRaises(ValueError):
+        with self.assertRaises(TypeError):
             self.experiment_type.update([1])
-        with self.assertRaises(ValueError):
+        with self.assertRaises(TypeError):
             self.experiment_type.update('this is not a dict')
 
     def test_nondefaults(self):
@@ -716,7 +735,7 @@ class DatapathTests (unittest.TestCase):
         # merge paths 1..3
         path1.merge(path2).merge(path3)
         self.assertNotEqual(path1.uri, original_uri, "Merged path's URI should have changed from its original URI")
-        self.assertEqual(path1.context.name, path3.context.name, "Context of merged paths should equal far right-hand path's context")
+        self.assertEqual(path1.context._name, path3.context._name, "Context of merged paths should equal far right-hand path's context")
         self.assertGreater(len(path1.Experiment.entities()), 0, "Should have returned results")
 
     def test_compose_paths(self):
@@ -731,7 +750,7 @@ class DatapathTests (unittest.TestCase):
         self.assertNotEqual(path.uri, path1.uri, "Composed path URI should not match the first path URI")
         self.assertEqual(path1.uri, original_uri, "First path was changed")
         self.assertNotEqual(path.uri, original_uri, "Merged path's URI should have changed from its original URI")
-        self.assertEqual(path.context.name, path3.context.name, "Context of composed paths should equal far right-hand path's context")
+        self.assertEqual(path.context._name, path3.context._name, "Context of composed paths should equal far right-hand path's context")
         self.assertGreater(len(path.Experiment.entities()), 0, "Should have returned results")
 
 
